@@ -1,5 +1,5 @@
 /* wmnetselect - Window Maker Netscape Selection dock app
-   wmnetselect version 0.8    Last Modification:  22 Apr 1999
+   wmnetselect version 0.85    Last Modification:  8 Aug 2000
    
    Code canabalized from asclock/wmclock/wmmail.
    Thanks to respective authors of those apps, Beat Christen,
@@ -7,7 +7,7 @@
    making their code available.
    
    All errors herein are made by Patrick Hill (apathos@bham.net)
-   This particular incarnation Copyright (c) 1999 Patrick Hill
+   This particular incarnation Copyright (c) 2000 Patrick Hill
    
  * 	This program is free software; you can redistribute it and/or modify
  * 	it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@ static Window MozWin = None;
 #include "netscape.xpm"
 #include "negative.xpm"
 
-#define VERSION 0.8
+#define VERSION 0.85
 #define DEBUG 0
 
 /* default executable is netscape, may be overridden by command line parameter */
@@ -63,6 +63,7 @@ static Window MozWin = None;
 #define DOUBLE_CLICK_TIME    250       /* double click latency (ms)  */
 #define MW_EVENTS   (ExposureMask | ButtonPressMask | StructureNotifyMask)
 #define FALSE 0
+#define NEWLINE_CHAR "\n"
 
 Display *dpy;
 Window Root;
@@ -86,6 +87,15 @@ typedef struct _XpmIcon {
     XpmAttributes attributes;
 }        XpmIcon;
 
+typedef struct query_target {
+  char * name;
+  char * pre;
+  char * post;
+} query_target_type ;
+int Where;
+
+int Raise;   /* Raise window? */
+
 XpmIcon nets, negt;
 XpmIcon visible;
 
@@ -99,8 +109,10 @@ void InsertTime();
 int ProcessSelection (char *selection, Display *dpy, int NewWin, int dontask,
     int noproto, int quiet, int button);
 int Selection2Bookmark (char *prefix, char *select, Window win);
-int URLSelection (char *prefix, char *select, Window win, int NewWin);
-int Search4Selection (char *select, Window win, int NewWin);
+int URLSelection (char *prefix, char *select, Window win, int NewWin, int raise);
+int Search4Selection (char *select, Window win, int NewWin, int raise, int where);
+int query_target_at (char *name);
+int killchar(char *str, char *killchar);
 void strim(char * str);
 void transform(char * str);
 char *encode (char *url, int button);
@@ -113,15 +125,68 @@ static int SendCommand (Display *dpy, Window win, const char *command);
 
 static char *help_message[] = {
     "where options include:",
-    " -dontask                Don't prompt for URL if there is no X selection.",
-    " -exe <program>          program to start on double-click, defaults to netscape.",
-    " -onstartup              start netscape when wmnetselect is started.",
-    " -noprotocol             Don't add http or ftp or mailto to URL.",
-    " -position [+|-]x[+|-]y  position of wmnetselect.",
-    " -quiet                  disable audible error indication.",
+    " -a <Y|n>                Ask for URL if there is no X selection.",
+    "                         Defaults to y",
+    " -e <program>            Executable to start on left button double-click.",
+    "                         Defaults to netscape",
+    " -i <Y|n>                Infer the protocol if the selection has none.",
+    "                         Defaults to y",
+    " -p <[+|-]X[+|-]Y>       X/Y Position of wmnetselect.",
+    "                         Defaults to +0+0",
+    " -q <y|N>                Quiet:  disable audible error indication.",
+    "                         Defaults to n",
+    " -r <Y|n>                Raise (deiconify) the <program> window.",
+    "                         Defaults to y",
+    " -s <y|N>                Start <program> when wmnetselect is started.",
+    "                         Defaults to n",
+    " -w <search engine>      Which search engine to use on right button click:",
+    "                         google, altavista, yahoo, excite, hotbot, lycos,",
+    "                         northernlight, alltheweb, or raging",
+    "                         Defaults to google",
     NULL
 };
 
+static query_target_type query_targets[] = {
+  { "google",
+    "http://www.google.com/search?q=",
+    "&num=30"
+  },
+  { "altavista",
+    "http://www.altavista.com/cgi-bin/query?pg=q&text=yes&what=web&fnt=d&q=",
+    ""
+  },
+  { "excite",
+    "http://search.excite.com/search.gw?search=",
+    "&start=0&showSummary=true&perPage=30"
+  },
+  { "hotbot",
+    "http://hotbot.lycos.com/?MT=",
+    "",
+  },
+  { "lycos",
+    "http://www.lycos.com/srch/?query=",
+    "",
+  },
+  { "northernlight",
+    "http://www.northernlight.com/nlquery.fcg?qr=",
+    "&cb=0&us=025"
+  },
+  { "alltheweb",
+    "http://alltheweb.com/cgi-bin/asearch?type=all&query=",
+    ""
+  },
+  { "yahoo",
+    "http://ink.yahoo.com/bin/query?p=",
+    ""
+  },
+  { "raging",
+    "http://ragingsearch.altavista.com/cgi-bin/query?q=",
+    "&nbq=50"
+  },
+};
+
+static int qt_num = sizeof(query_targets) / sizeof(query_target_type);
+
 void usage() {
     char **cpp;
 
@@ -135,9 +200,19 @@ void usage() {
     exit(1);
 }
 
+int query_target_at (char *name) {
+    int i;
+    for (i=0; i < qt_num; i++) {
+        if (!strcmp(query_targets[i].name,name))
+	    return i;
+    }
+    fprintf (stderr, "Unknown search engine (%s). Defaulting to (%s).\n",
+		name, query_targets[0].name);
+    return 0;
+}
 
 int main(int argc,char *argv[]) {
-    static Time last_left_mouse_click   = -1;
+    static Time last_left_mouse_click = -1;
     int i;
     unsigned int borderwidth ;
     char *display_name = NULL;
@@ -159,7 +234,8 @@ int main(int argc,char *argv[]) {
     int ret = 25;
     
     cnt = dontask = onstart = noproto = quiet = 0;
-
+    Raise = 0;
+    
     ProgName = argv[0];
     Geometry = "";
 
@@ -167,32 +243,57 @@ int main(int argc,char *argv[]) {
     strcpy (Execute, EXECUTABLE);
 
     /* Parse command line options */
-    for(i=1;i<argc;i++) {
-        char *arg= argv[i];
+    for (i=1; i<argc; i++) {
+        char *arg = argv[i];
 
         if (arg[0] == '-') {
-            switch(arg[1]) {
+            switch (arg[1]) {
+            case 'a':
+                if (++i >= argc) usage();
+                if (tolower(*argv[i]) == 'n')
+	                dontask=1;	/* if no selection, ask */
+                continue;
             case 'd':
-                dontask=1;	/* if no selection, ask */
+                if (++i >= argc) usage();
+		display_name = argv[i];
                 continue;
             case 'e':
-                if(++i >=argc) usage();
+                if (++i >= argc) usage();
                 strcpy(&Execute[0], argv[i]);
                 continue;
-            case 'o':
-                onstart=1;	/* start it at startup */
-                continue;
-            case 'n':
-                noproto=1;	/* don't infer protocol */
+            case 'i':
+            	if (++i >= argc) usage();
+                if (tolower(*argv[i]) == 'n')
+                	noproto=1;	/* don't infer protocol */
                 continue;
             case 'p':
-                if(++i >=argc) usage();
+                if (++i >= argc) usage();
                 Geometry = argv[i];
                 continue;
-            case 's':
             case 'q':
-                quiet=1;	/* no XBells */
+            	if (++i >= argc) usage();
+                if (tolower(*argv[i]) == 'y')
+                	quiet=1;	/* no XBells */
                 continue;
+            case 'r':
+            	if (++i >= argc) usage();
+                if (tolower(*argv[i]) == 'n')
+                	Raise=1;	/* don't raise window */
+                continue;
+            case 's':
+            	if (++i >= argc) usage();
+                if (tolower(*argv[i]) == 'y')
+	                onstart=1;	/* start it at startup */
+                continue;
+	    case 'w':
+		if (++i >= argc) usage();
+		Where = query_target_at(argv[i]); 
+		continue;
+	    case 'v':
+		fprintf (stderr,"\n%s version %4.2f\n\n",
+            		ProgName, VERSION);
+		exit(0);
+		continue;	/* for consistency only */
             default:
                 usage();
             }
@@ -404,9 +505,9 @@ static void CreatePixmap(Display *dpy, Window Root, char **data, XpmIcon* icon)
     int ret;
 
     icon->attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions
-                                   | XpmExactColors | XpmCloseness);
+				   | XpmExactColors | XpmCloseness);
     icon->attributes.exactColors=False;
-    icon->attributes.closeness=40000;
+    icon->attributes.closeness=65636;
     ret = XpmCreatePixmapFromData(dpy, Root, data, &(icon->pixmap),
                                   &(icon->mask), &(icon->attributes));
     if(ret != XpmSuccess) {
@@ -471,10 +572,21 @@ Pixel GetColor(char *name) {
 int ProcessSelection (char *selection, Display *dpy, int NewWin, int dontask,
         int noproto, int quiet, int button) {
     char *prefix, *select;
-    int err;
+    int err, raise;
     Window win;
 
     strim (selection);
+
+    /* If we're using the middle button, we're trying to process
+     * a URL. If so, then we have no need for spaces in the middle
+     * of the URL. Squashing these spaces allows us to process URLs
+     * that are broken across one or more lines.
+     */
+    if ( button != RIGHT_BUTTON ) {
+        killchar(selection, " ");
+        killchar(selection, NEWLINE_CHAR);
+    }
+
     select = encode (selection, button);
     if (select == NULL) {
         fprintf (stderr, "Could not allocate memory\n");
@@ -484,6 +596,7 @@ int ProcessSelection (char *selection, Display *dpy, int NewWin, int dontask,
     /* transform keywords like slashdot, windowmaker, etc. */
     transform (select);
     
+    raise = Raise;
     /* This block determines how to handle Null selections.  */
     if (strlen (select) == 0) {
         if (dontask) {
@@ -493,6 +606,7 @@ int ProcessSelection (char *selection, Display *dpy, int NewWin, int dontask,
         } else {
 	    NewWin = 0;
 	    noproto = 1;
+	    raise = 0;  /* Always raise if selection is NULL */
  	}
     }
     prefix = protocol (select, noproto);
@@ -502,10 +616,10 @@ int ProcessSelection (char *selection, Display *dpy, int NewWin, int dontask,
         err = Selection2Bookmark (prefix, select, FindMozWin(dpy));
         break;
     case MIDDLE_BUTTON:
-        err = URLSelection (prefix, select, FindMozWin(dpy), NewWin);
+        err = URLSelection (prefix, select, FindMozWin(dpy), NewWin, raise);
         break;
     case RIGHT_BUTTON:
-        err = Search4Selection (select, FindMozWin(dpy), NewWin);
+        err = Search4Selection (select, FindMozWin(dpy), NewWin, raise, Where);
         break;
     }
     if (select) free (select);
@@ -548,7 +662,7 @@ int Selection2Bookmark (char *prefix, char *select, Window win) {
     return error;
 }
 
-int URLSelection (char *prefix, char *select, Window win, int NewWin) {
+int URLSelection (char *prefix, char *select, Window win, int NewWin, int raise) {
     char *netcommand = NULL;
     int error;
     
@@ -565,18 +679,22 @@ int URLSelection (char *prefix, char *select, Window win, int NewWin) {
                  prefix, select);
         error = system (netcommand);
     } else /* win != None */ {
-        netcommand = (char *) malloc(sizeof(char) * (strlen(select) + 27));
+        /* netcommand = (char *) malloc(sizeof(char) * (strlen(select) + 27));
+	*/
+        netcommand = (char *) malloc(sizeof(char) * (strlen(select) + 35));
         if (netcommand == NULL) {
             fprintf (stderr, "Could not allocate memory\n");
             return -1;
         }
-        sprintf (netcommand, "openURL(%s%s%s)",
-                 prefix, select, NewWin ? ",newwindow" : "");
+        sprintf (netcommand, "openURL(%s%s%s%s)",
+                 prefix, select, NewWin ? ",new-window" : "",
+		 raise ? ",noraise" : "");
 
         error = SendCommand(dpy, win, netcommand);
     }
-    /* printf ("\nerr = <%d>, netcommand=<%s> ",error, netcommand);  */
-    if (netcommand) free (netcommand);
+    /* printf ("\nerr = <%d>, netcommand=<%s> ",error, netcommand); */
+    if (netcommand) 
+		free (netcommand);
     return error;
 }
 
@@ -584,33 +702,36 @@ int URLSelection (char *prefix, char *select, Window win, int NewWin) {
  * Send selection to the search engine as a query.
  */
  
-int Search4Selection (char *select, Window win, int NewWin) {
-    static char *google = "http://www.google.com/search?q=";
-    static char *numhits = "&num=30";
+int Search4Selection (char *select, Window win, int NewWin, int raise, int where) {
+    query_target_type target = query_targets[where];
     char *netcommand = NULL;
     int error;
     
     if (win == None) {
         /* No netscape window, start one.  */
-        /* 2>/dev/null + spaces + & + "" + google + numhits = 55 */
-        netcommand = (char *)malloc (sizeof(char) * (strlen(Execute)
-	    + strlen(select) + 56));
+        /* 2>/dev/null + spaces + & + "" = 17 */
+        netcommand = (char *)
+	   malloc (sizeof(char) * (strlen(Execute)
+	    + strlen(target.pre) + strlen(select) + strlen(target.post) + 18));
         if (netcommand == NULL) {
             fprintf (stderr, "Could not allocate memory\n");
             return -1;
         }
         sprintf (netcommand, "%s \"%s%s%s\" 2>/dev/null &", Execute,
-                 google, select, numhits);
+                 target.pre, select, target.post);
         error = system (netcommand);
     } else /* win != None */ {
-        /* strlen ("openURL(,newwindow)") + google + numhits = 57 */
-        netcommand = (char *) malloc(sizeof(char) * (strlen(select) + 58));
+        /* strlen ("openURL(,new-window)") = 20 */
+        netcommand = (char *)
+	   malloc(sizeof(char) * (
+	      strlen(target.pre) + strlen(select) + strlen(target.post) + 28));
         if (netcommand == NULL) {
             fprintf (stderr, "Could not allocate memory\n");
             return -1;
         }
-        sprintf (netcommand, "openURL(%s%s%s%s)",
-                 google, select, numhits, NewWin ? ",newwindow" : "");
+        sprintf (netcommand, "openURL(%s%s%s%s%s)",
+                 target.pre, select, target.post, NewWin ? ",new-window" : "",
+		 raise ? ",noraise" : "");
 
         error = SendCommand(dpy, win, netcommand);
     }
@@ -618,6 +739,43 @@ int Search4Selection (char *select, Window win, int NewWin) {
     return error;
 }
 
+/* killchar()
+ *  
+ *  Arguments: pointer to a character string
+ *  Purpose  : Removes newlines from the string.
+ *  Returns  : number of newlines removed
+ *  [Contributed by bigdan@kreft.net]
+ */
+
+int killchar(char *str, char *killchar) {
+    char *newstr;
+    char *next_chunk;
+    int killed = 0;         // Number of newlines removed
+    
+    /* Just leave if there aren't any newlines to mess with. */
+    if (NULL == str || strchr(str, *killchar) == (char *) NULL ) {
+        return(0);
+    }
+    
+    /* Allocate memory for buffer. */
+    newstr = (char *) malloc(sizeof(char) * strlen(str));
+    
+    /* Get the chunk before the first newline and append it to newstr.  */
+    next_chunk = strtok(str, killchar);
+    strcpy(newstr, next_chunk);
+    
+    /* Get and append all remaining chunks delimited by newlines.  */
+    while ( (next_chunk = strtok(NULL, killchar)) != (char *)NULL ) {
+        strncat(newstr, next_chunk, strlen(next_chunk));
+        ++killed;
+    }
+
+    strncpy(str, newstr, strlen(newstr)+1);
+    free(newstr);
+    
+    return(killed);
+}
+
 /* Trim whitespace from both ends of str */
 
 void strim(char *str) {
@@ -645,6 +803,15 @@ void transform (char *str) {
         strcat (str, ".org");
     else if (strcasecmp (str, "freshmeat") == 0)
         strcat (str, ".net");
+
+/*   need to add www.prefix to these:
+    else if (strcasecmp (str, "gnu") == 0)
+        strcat (str, ".org");
+    else if (strcasecmp (str, "gnome") == 0)
+        strcat (str, ".org");
+    else if (strcasecmp (str, "kde") == 0)
+        strcat (str, ".org");
+*/
 }
 
 /*
@@ -662,7 +829,7 @@ char *encode (char *url, int button) {
     static char *punctuation = "#?&+$%\":<>/.=!*'(),{};|\\^~[]`";
     
     if (NULL == url || (len = strlen(url)) == 0) {
-        ncoded = (char *) malloc(sizeof(char) * 2);
+        ncoded = (char *) malloc(sizeof(char) );
 	*ncoded = '\0';
 	return (ncoded);
     }
@@ -673,7 +840,7 @@ char *encode (char *url, int button) {
     i=j=0;
 
     /* allocate extra chars for ".org" or ".net"   */
-    ncoded = (char *) malloc(sizeof(char) * (len + 5));
+    ncoded = (char *) malloc(sizeof(char) * len * 3);
     if (ncoded == NULL) {
         fprintf (stderr, "Could not allocate memory\n");
         return NULL;
@@ -682,9 +849,10 @@ char *encode (char *url, int button) {
     while (1) {
 	if (strchr (uns, url[j])) {
 	           sprintf (esc, "%2x", url[j]);
-            if ((ncoded = (char *) realloc(ncoded, sizeof(char) * (len + 4)))
+        /*   if ((ncoded = (char *) realloc(ncoded, sizeof(char) * (len + 4)))
                         == NULL)
                 return NULL;
+	*/
             ncoded[i++]='%';
             ncoded[i++]=esc[0];
             ncoded[i++]=toupper(esc[1]);
@@ -720,6 +888,11 @@ char *protocol (char *url, int noproto) {
     /* contains a '@' but no '/'  */
     if (!strchr(url, '/') && strchr(url, '@'))
          return (strcpy (ptcol, "mailto:"));
+
+    /* first character is a '/', then it must be a file */
+    if (*url == '/')
+         return (strcpy (ptcol, "file:"));
+
     return (strcpy (ptcol, "http://"));
 }
 
